"""Pop :term:`VLAN` product block that has all parameters of a subscription throughout its lifecycle."""

from ipaddress import IPv4Network, IPv6Network
from typing import Annotated, TypeVar

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
from pydantic import AfterValidator
from pydantic_forms.types import strEnum
from pydantic_forms.validators import validate_unique_list
from typing_extensions import Doc

from gso.products.product_blocks.lan_switch_interconnect import (
    LanSwitchInterconnectBlock,
    LanSwitchInterconnectBlockInactive,
    LanSwitchInterconnectBlockProvisioning,
)

T = TypeVar("T")


class LayerPreference(strEnum):
    """Enumerator for the different types of layer preferences."""

    L2 = "L2"
    L3 = "L3"


PortList = Annotated[list[T], AfterValidator(validate_unique_list), Doc("A list of unique ports.")]


class PopVlanPortBlockInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="PopVlanPortBlock"
):
    """An inactive Pop :term:`VLAN` port."""

    port_name: str | None = None
    port_description: str | None = None
    tagged: bool | None = None


class PopVlanPortBlockProvisioning(PopVlanPortBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A Pop :term:`VLAN` port that is being provisioned."""

    port_name: str | None
    port_description: str | None
    tagged: bool | None


class PopVlanPortBlock(PopVlanPortBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """An active Pop :term:`VLAN` port."""

    port_name: str
    port_description: str
    tagged: bool


class PopVlanBlockInactive(
    ProductBlockModel,
    lifecycle=[SubscriptionLifecycle.INITIAL],
    product_block_name="PopVlanBlock",
):
    """A Pop :term:`VLAN` that's currently inactive, see :class:`PopVlanBlock`."""

    vlan_id: int
    pop_vlan_description: str | None = None
    lan_switch_interconnect: LanSwitchInterconnectBlockInactive
    ports: PortList[PopVlanPortBlockProvisioning]
    layer_preference: LayerPreference
    ipv4_network: IPv4Network | None = None
    ipv6_network: IPv6Network | None = None


class PopVlanBlockProvisioning(PopVlanBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A Pop :term:`VLAN` that's currently being provisioned, see :class:`PopVlanBlock`."""

    vlan_id: int
    pop_vlan_description: str | None
    lan_switch_interconnect: LanSwitchInterconnectBlockProvisioning
    ports: PortList[PopVlanPortBlockProvisioning]
    layer_preference: LayerPreference
    ipv4_network: IPv4Network | None
    ipv6_network: IPv6Network | None


class PopVlanBlock(PopVlanBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A Pop :term:`VLAN` that's currently deployed in the network."""

    #: The :term:`VLAN` ID of the Pop :term:`VLAN`.
    vlan_id: int
    #: The description of the Pop :term:`VLAN`.
    pop_vlan_description: str
    #: The :term:`LAN` Switch Interconnect that this Pop :term:`VLAN` is connected to.
    lan_switch_interconnect: LanSwitchInterconnectBlock
    #: The ports of the Pop :term:`VLAN`.
    ports: PortList[PopVlanPortBlock]  # type: ignore[assignment]
    #: The level of the layer preference for the Pop :term:`VLAN` (L2 or L3).
    layer_preference: LayerPreference
    #: IPv4 network for the Pop :term:`VLAN` if layer preference is L3.
    ipv4_network: IPv4Network | None
    #: IPv6 network for the Pop :term:`VLAN` if layer preference is L3.
    ipv6_network: IPv6Network | None
